/**************************************************************************************
(SAS 9.13 or higher) and (Winsteps 3.64 or higher) are required to run this program!!
The purpose of this SAS program is to format a Winsteps control file with a partial 
credit. The input file should be a string indicating the maximum score of each item. 
For example:                                                                 
111111111111111111111111111234                                                     
In this example, the last three items are partial-credit items, which are worth two, three, 
and four points, respectively. This string is stored in a file named 'input_example.txt'. 
If all items are binary (1, 0), then this program will output a binary key.
The output is named 'output_example.con', which is a program ready to be run in Winsteps.
This SAS program utilies SAS macros. Users can simply input values into three macro 
variables in the beginning and the submit the code to SAS. Customization of the output
may be needed.
Written by Chong Ho Yu, Ph.D., Email: chonghoyu@yahoo.com                          
***************************************************************************************/                                                    

/* input the value into the following global macro variables */
/* Where is your working directory? */
%LET path = c:\IRT\ ;
/* what is the name of the input file? */
%LET input = input_example.txt;

/* No change is needed below this line */

/* Count how many items are there in the key */
data countkey; infile "&path&input" truncover;
input textstring $999. ;
countkey=length(textstring); 
call symputx("lastkey", countkey);
run;

/* Import the score string individually */
data pci; infile "&path&input";
input (q1-q&lastkey) (1.0); run;

/* Find the maximum score among all items */
proc transpose data=pci out=transpci; run; 
proc sort data=transpci; by descending col1; run;
data transpci; set transpci;
if _N_ = 1 then call symputx("max", col1); run;

/* Create a reverse base number to generate the partial credit scores from the key counter */ 
%LET reversebase = %eval(&max + 1);

/* Create multiple rows for the partial credit key by setting the same dataset 'pci' multiple
times, based upon the maximum score. In the input_example.txt, the maximum score is 4 and
thus 'pci' is reset four times */ 
%macro setpci;
data pci_all; set
%do i = 1 %to &max;
        pci 
%end;
;
run;
%mend setpci;

%setpci;

/* Add the key counter */
data pci_all; set pci_all;
key = "key";
keycounter = _N_;
equal = "=";
run;

/* Generate the partial credit key. Since the symbol "*" is not numeric, SAS log will show 
many error messages. Please ignore those messages. The output will be fine. */
data pci_key; set pci_all;
array q(*) q1-q&lastkey;
array w(*) $ w1-w&lastkey ;
do i = 1 to &lastkey;
		if q(i) = 1 and keycounter ne &max then w(i) = "*";
		else if q(i) > 1 then w(i) = &reversebase - keycounter;
		else w(i) = q(i);
		if w(i) > q(i) then w(i) = "*";
		
end;
drop i q1-q&lastkey;
run;

/* Output the control file for Winsteps. This control file will include the partial credit key.
Users need to customize the control file such as adding the raw data filename */
filename out  "&path\output_example.con" lrecl=1028;

data _null_; 
file out;
put "&INST";
put "NI=&lastkey ; Number of items";
put "ITEM1=5 ; Position of where the first item begins.";
put "CODES=012345678 ; Valid answers";  

/* Output the partial credit key. Afterwards, you can copy and paste the file "pci.key"
into the Winsteps control file */
data pci_key; set pci_key;
file out mod;
put 
@1 key @4 keycounter @5 equal @6 (w1-w&lastkey) (:$16.0 +(-1));run;

data _null_;
file out mod;
put "GROUPS=0";
put "MODELS=R ; Partial Credit model is based on the Rating Scale Model."; 
put "UPMEAN=0 ; Set the mean of examinees' ability to 0."; 
run;

data pci_scr; set pci_key;
keep keycounter; run;
proc sort data=pci_scr; by descending keycounter;
proc transpose data=pci_scr out=pci_scr2; run;

data pci_scr2; set pci_scr2;
file out mod;
put "KEYSCR=" (_numeric_) (1.0); run;

data _null_;
file out mod;
put "NAME1=1 ; The position of where the subject ID starts.";
put "NAMELEN=3 ; Length of subject ID";
put "; output file names";
put "; sf is the step function file for partial credits.";
put "; que is the question file for item parameters.";
put "; per is the person file for person theta.";
put "SFILE=example.sf";
put "IFILE=example.que";
put "PFILE=example.per";
put "; Prefix for person and item. They are arbitrary.";
put "PERSON=S";
put "ITEM=I";
put "DATA=example.dat ; Name of the raw data file";
put "; You can insert the list of item ID between &end and end names or leave it blank";
put "&END";
put " ";
put "END NAMES";
run;



